<?php

declare(strict_types=1);

namespace App\DTO\Factory;

use App\DTO\WeatherAtTimePoint;

class WeatherAtTimePointFactory
{
    public function createFromUntrustedDataWithTz(\DateTimeImmutable $mutableDate, float $min, float $max, ?float $value): WeatherAtTimePoint
    {
        $timeZone = new \DateTimeZone(date_default_timezone_get());
        $mutableDate = \DateTime::createFromImmutable($mutableDate);
        $mutableDate->setTimezone($timeZone);
        return new WeatherAtTimePoint(\DateTimeImmutable::createFromMutable($mutableDate), $min, $max, $value);
    }

    public function createFromUntrustedWithDateYYYYmmdd(string $date, float $min, float $max, ?float $value): WeatherAtTimePoint
    {
        $parsedDate = \DateTimeImmutable::createFromFormat('Y-m-d H:i:s', $date . " 00:00:00");
        if ($parsedDate === false) {
            throw new \InvalidArgumentException("$date is not in format Y-m-d");
        }
        return new WeatherAtTimePoint(
            $parsedDate,
            $min,
            $max,
            $value,
        );
    }
}
