<?php

declare(strict_types=1);

namespace App\Services;

use App\DTO\Factory\GEOPointFactory;
use App\DTO\GEOPoint;

class GeoPointResolverService
{
    /**
     * @var array<string, GEOPoint>
     */
    protected readonly array $citiesWithPoints;

    /**
     * @param array<string, array<string, string>> $geoPointsWithCities
     */
    public function __construct(
        array $geoPointsWithCities,
        protected readonly GEOPointFactory $geoPointFactory,
    ) {
        $citiesWithPoints = [];
        foreach ($geoPointsWithCities as $cityName => $cityDef) {
            if (!preg_match('/^[\w ]+$/u', $cityName)) {
                throw new \InvalidArgumentException("Invalid city $cityName -- Name must be [\w ]+");
            }

            $geoPoint = $this->geoPointFactory->createFromString(
                $cityDef['lat'],
                $cityDef['lon']
            );

            if ($geoPoint === null) {
                throw new \InvalidArgumentException("Can\'t create GEOPoint from " . $cityDef['lat'] . "/" . $cityDef['lon']);
            }

            $citiesWithPoints[$cityName] = $geoPoint;
        }
        $this->citiesWithPoints = $citiesWithPoints;
    }

    /**
     * @return array<string, GEOPoint>
     */
    public function getCitiesWithPoints(): array
    {
        return $this->citiesWithPoints;
    }

    public function getCityNameFromGeoPoint(GeoPoint $searchedGeoPoint): ?string
    {
        foreach ($this->citiesWithPoints as $cityName => $geoPoint) {
            if ($geoPoint->equals($searchedGeoPoint)) {
                return $cityName;
            }
        }

        return null;
    }

    public function getGeoPointFromCityName(string $searchedCityName): ?GEOPoint
    {
        if (!preg_match('/^[\w ]+$/u', $searchedCityName)) {
            return null;
        }

        foreach ($this->citiesWithPoints as $cityName => $geoPoint) {
            if ($cityName === $searchedCityName) {
                return $geoPoint;
            }
        }

        return null;
    }
}
