<?php

declare(strict_types=1);

namespace App\Services\Granularity;

use App\DTO\WeatherAtTimePoint;
use App\Services\WeatherGranularityInterface;
use DateTimeImmutable;
use Nubium\Exception\ThisShouldNeverHappenException;

class DayWeatherByDate implements WeatherGranularityInterface
{
    public function __construct()
    {
    }

    /**
     * @param array<WeatherAtTimePoint> $records
     * @return array<WeatherAtTimePoint>
     */
    public function calculateFromMultipleRecords(array $records): array
    {
        $weatherDataByDates = [];

        foreach ($records as $weatherDataAtTimePoint) {
            if (!isset($weatherDataByDates[$weatherDataAtTimePoint->date->format('Y-m-d')])) {
                $weatherDataByDates[$weatherDataAtTimePoint->date->format('Y-m-d')] = [];
            }
            $weatherDataByDates[$weatherDataAtTimePoint->date->format('Y-m-d')][] = $weatherDataAtTimePoint;
        }

        return $this->resetTimeToMidnight($this->flatizeWeatherData($weatherDataByDates));
    }

    /**
     * @param array<string, array<WeatherAtTimePoint>> $weatherData
     * @return array<int, WeatherAtTimePoint>
     */
    protected function flatizeWeatherData(array $weatherData): array
    {
        $flatizedWeatherData = [];
        foreach ($weatherData as $weatherDataAtTimePoints) {
            if (count($weatherDataAtTimePoints) === 1) {
                $flatizedWeatherData[] = $weatherDataAtTimePoints[0];
            } else {
                // calculate avg value, max for max, min for min
                $maxWeatherAtDate = PHP_INT_MIN;
                $minWeatherAtDate = PHP_INT_MAX;
                $valueWeatherDataPoints = 0;
                foreach ($weatherDataAtTimePoints as $weatherDataAtTimePoint) {
                    $maxWeatherAtDate = max($maxWeatherAtDate, $weatherDataAtTimePoint->max);
                    $minWeatherAtDate = min($minWeatherAtDate, $weatherDataAtTimePoint->min);
                    $valueWeatherDataPoints += $weatherDataAtTimePoint->value;
                }

                $flatizedWeatherData[] = new WeatherAtTimePoint(
                    $weatherDataAtTimePoints[0]->date,
                    $minWeatherAtDate,
                    $maxWeatherAtDate,
                    $valueWeatherDataPoints / count($weatherDataAtTimePoints)
                );
            }
        }

        return $flatizedWeatherData;
    }

    /**
     * @param array<WeatherAtTimePoint> $weatherData
     * @return array<int, WeatherAtTimePoint>
     */
    protected function resetTimeToMidnight(array $weatherData): array
    {
        $resetWeatherAtTimePoints = [];
        foreach ($weatherData as $weatherDataAtTimePoints) {
            $date = DateTimeImmutable::createFromFormat('Y-m-d H:i:s', $weatherDataAtTimePoints->date->format('Y-m-d 00:00:00'));

            if ($date === false) {
                throw new ThisShouldNeverHappenException("Can't create date from date" . $weatherDataAtTimePoints->date->format('Y-m-d 00:00:00'));
            }

            $weatherDataAtTimePoints = new WeatherAtTimePoint(
                $date,
                $weatherDataAtTimePoints->min,
                $weatherDataAtTimePoints->max,
                $weatherDataAtTimePoints->value
            );

            $resetWeatherAtTimePoints[] = $weatherDataAtTimePoints;
        }

        return $resetWeatherAtTimePoints;
    }
}
